/* polyathlon.c 
// program by Dale Heatherington  started April 23, 2009

// Version 2.0   June 8, 2009
// Modified for ATMEGA644 CPU.  This chip has 64K Program storage and 4K RAM.
// Note: This will not compile for the old ATMEGA8535 anymore.


Compiled with AVR-GCC on Apple Mac Pro using Crosspack-AVR
 
Note: CPU type defined in Makefile but it must be a ATMEGA644
      Brownout protection mode must be set in the low fuse byte to prevent EEPROM corruption.
	  This is done in the makefile in the "pgm" section for the avrdude programmer
	  
	  Initially the line sensor photocell array must be calibrated on a white surface
	  by setting the mode switch to 8 and turning on power for 10 seconds.  Values are stored in EEPROM.
	  

This is a work in progress is may be incomplete or infested with bugs.
It has been tweeked to work with the hardware and sensors available and will 
probably not function on any other hardware.  It is presented for educational
purposes.



;;----------------------------------------------------------------------
;;This program is free software; you can redistribute it and/or modify  
;;it under the terms of the GNU General Public License as published by      
;;the Free Software Foundation; either version 2 of the License, or         
;;(at your option) any later version.                                       
;;
;;This program is distributed in the hope that it will be useful,       
;;but WITHOUT ANY WARRANTY; without even the implied warranty of            
;;MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             
;;GNU General Public License for more details.                              
                                                                          
;;You should have received a copy of the GNU General Public License     
;;along with this program; if not, write to the Free Software               
;;Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA
;;------------------------------------------------------------------------

*/
 



#include <avr/io.h>
#include <stdio.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>

#include <avr/eeprom.h>
#include <stdlib.h>
#include <string.h>
#include <avr/wdt.h>

#define F_CPU 16000000UL  // 16 MHz CPU Clock
#include <util/delay.h>


/* I/O and bit operation macros that have were 
   removed from late versions of avr-libc */

#define  _BV(bit)   (1 << (bit))
#define  inb(sfr)   _SFR_BYTE(sfr)
#define  inw(sfr)   _SFR_WORD(sfr)
#define  outb(sfr, val)   (_SFR_BYTE(sfr) = (val))
#define  outw(sfr, val)   (_SFR_WORD(sfr) = (val))
#define  cbi(sfr, bit)   (_SFR_BYTE(sfr) &= ~_BV(bit))
#define  sbi(sfr, bit)   (_SFR_BYTE(sfr) |= _BV(bit))

#define  loop_until_bit_is_set(sfr, bit)   do { } while (bit_is_clear(sfr, bit))
#define  loop_until_bit_is_clear(sfr, bit)   do { } while (bit_is_set(sfr, bit))
#define  outp(val, sfr)   outb(sfr, val)
#define  inp(sfr)   inb(sfr)
#define  BV(bit)   _BV(bit)
/*---------------------------------------------------------------------------------*/

#define WDTO_250MS 4        /* Watchdog timer value = 250ms */

#define TRUE 1
#define FALSE 0

#define MODE_STOP 0
#define REV 0
#define FWD 1
#define CW 2
#define CCW 1

#define LEFT 2
#define RIGHT 1
#define INTERSECTION 3
#define BOTH 3
#define NONE 0

#define MAX_FWD_SPEED -127
#define MAX_REV_SPEED 127

#define MAXTIME 180

//Threshold for IR edge detectors used as line detectors
//White is above this number, black below.
#define ET 0xb0



//Counts per inch integer (actually 36.9)
#define CPI 37


/* Define string constants in FLASH program memory */
const char testquad[] PROGMEM = "TestQuad";
const char linefoll[] PROGMEM = "LineFoll";
const char beaconfinder[]  PROGMEM = "Beacon";
const char deadrec[]  PROGMEM = "DeadRec";
const char bulldoz[]  PROGMEM = "BullDoz";
const char linemaze[] PROGMEM = "LineMaze";
const char whitelevel[] PROGMEM = "WhiteLvl";
const char start[] PROGMEM = "Start";
const char facebeacon[] PROGMEM = "FaceBeac";



/*--- Motor control variables and defs ----- */

//Bit in PORTB that enables drive motor h-bridges
#define MOT_ENAB 4

int r_speed;		//Motor speed control variables
int l_speed;
int speed;
uint8_t m_current;	//Motor current stored here
volatile uint8_t timer_mstop;


//Prescalers used in timer interrupt to keep track of time.
uint8_t prescale ;
uint8_t prescale100;
uint8_t prescale1sec;
uint8_t prescaleMinute;

uint8_t timer1sec;
volatile unsigned int minutes;
volatile int run_seconds,timer_trip, timer_move;
volatile int timer_led_yellow, timer10ms, timer100us, timer_overcurrent, timer_pid, timer_delay;
volatile uint8_t rnd;

//Quadrature encoder data
volatile unsigned long odo_left, odo_right;
volatile  long trip_odo_right, trip_odo_left;
volatile long line_history_odo_right, line_history_odo_left;
volatile int speed_left, speed_right;
volatile uint8_t dir_left, dir_right;

#define TXBLEN 24
char tx_buf[TXBLEN];
volatile uint8_t tx_ptr,tx_ctr ;
volatile uint8_t txstate;


uint8_t debug[6];  	//Debug byte array - put data to be displayed on LCD here.

uint8_t last_sec;

uint8_t trip_counter;
uint8_t stop;
uint8_t sel_sw;
uint8_t edge_left, edge_right;  //Table edge detector analog values 0..255
int avg_contrast[4];

uint8_t   pot, motor_current;


struct Line {	
	int centroid;
	int contrast;
	uint8_t brightness;
	uint8_t present;
	uint8_t mean;
	uint8_t map;
	uint8_t width;
	uint8_t data[9];  //last byte is checksum for eeprom storage
};

struct Line line;

struct Beacon {
	int centroid;	
	uint8_t brightness;
	uint8_t left;
	uint8_t right;
	uint8_t maxbright;
	uint8_t minbright;
	
	
};

struct Beacon beacon;


struct IR {
	int centroid;
	uint8_t brightness;
	uint8_t contrast;
	uint8_t mean;
	uint8_t map;
	uint8_t data[6];
	
};

struct IR activeIR;

//PID data structure
struct PID {
	int kp;		//Porportional gain
	int p;		//Porportional
	int kd;		//deriviative gain
	int d;		//deriviative
	int ki;		//integral gain
	long i;		//integral
	int t;		//sampling time in milliseconds
	int ilim;  //Integrator high/low limits
	int u;		//updated pid output value
};

struct PID pid_line;		//Line follower PID structure instance
	

char sbuf[24];  //LCD bug tx buffer;

int samples[16];		//Holds 16 beacon sensor samples for correlator




// Analog to digital converter input defs
#define AD_LINE_FOLLOW 7
#define AD_POT  6
#define AD_EDGE_PULSE 4
#define UNUSED 5
#define AD_BEACON_RIGHT 3
#define AD_BEACON_LEFT 2
#define AD_IR_PULSE  1
#define AD_MOTOR_CURRENT 0


uint8_t irsensor[8];
//Define Active IR LEDs in sensor irsensor[] array
#define REV_RIGHT 2
#define REV_LEFT 3
#define FWD_LEFT 1
#define FWD_RIGHT 0


int getContrast(uint8_t , uint8_t*, uint8_t*  );
uint8_t getMap(uint8_t ,  uint8_t*, uint8_t );




//-----------------------------------------------------------------
// Actual code begins here.
//-----------------------------------------------------------------

// Waste time.
// t is in 10 ms intervals
void delay(int t)
{
	timer_delay = t;
	while(timer_delay > 0);
	
}

//Wait n times 100us
void waitx100us(int n)
{
   timer100us = n ;
   while( timer100us > 0);
   
}


//------------------------------------------------------------------

//Convert 4 bit value to ascii hex format 0..f

char nibble(uint8_t c)
{	
	c = c & 0x0f;
	if(c < 10) return c + 0x30;   	//0..9
	return c + 0x37;		//A..F
}
	
//Convert 8 bit value to ascii string hex format 00..ff in buffer *cp
//A NULL will be appended to the end of the 2 digit string.
//Supplied buffer must be at least 3 bytes long
void bin2hex(char *cp, uint8_t i)
{  uint8_t t;

	t = i >> 4;
	cp[0] = nibble(t);
	t = i & 0x0f;
	cp[1] = nibble(t);
	cp[2] = 0x00;
		
}
//-------------------------------------------------------------------------


int hard_limit_range(int x, int lim_hi, int lim_lo)
{
   if(x < lim_lo) x = lim_lo; 
   if(x > lim_hi) x = lim_hi; 
   return x;

}

//---------------------------------------------------------------------------

/* Read 10 bit AD converter but return 
   only 8 bits discarding low 2 bits. */

uint8_t get_adc8(uint8_t chan)
{  
    uint16_t result;
   
    ADMUX = chan;					//Select channel
    sbi(ADCSRA,ADSC);                //Start conversion     
    while( bit_is_clear(ADCSRA,ADIF)); //Wait for ADC output registers to be loaded
    cli();   
	result = ADCW;			//Get 10 bit value
    sei();
    return result >> 2;     //Return 8 bit value
}
//---------------------------------------------------------------------------
//Compute simple checksum of array of unsigned bytes.
uint8_t checksum(uint8_t *p, int n)
{   int i;
    uint8_t cksum = 0;
	for(i=0;i<n;i++) cksum += p[i] ;
	return cksum;
}


//---------------------------------------------------------------------------
//Read an active IR seonsor
//Inputs: adr = address of IR LED 0..7
//
uint8_t readActiveIR(uint8_t adr)
{	uint16_t result;
	adr = adr & 7;	
	
	if(adr & 1) sbi(PORTC,0); else cbi(PORTC,0);  //Address 1 of 8 IR LEDs
	if(adr & 2) sbi(PORTC,1); else cbi(PORTC,1);
	if(adr & 4) sbi(PORTC,2); else cbi(PORTC,2);
		
	cbi(PORTC,3);		//Enable selected IR LED and wait 30uS
	_delay_us(30);
	
	// Now either read the object detector or edge detector AD converter channel...
	// ...depending on which LED we are firing.  
	// Addresses 6 and 7 are for the Edge Detectors.
	// Addresses 0..5 are object detector LEDs.
	if(adr < 6) ADMUX = AD_IR_PULSE; 
		else ADMUX = AD_EDGE_PULSE;
		
	sbi(ADCSRA,ADSC);						//Start conversion now
	cli();
	_delay_us(20);							//Wait about 20uS for sample and hold
	sei();
	sbi(PORTC,3);							//IR pulse off
	while( bit_is_clear(ADCSRA,ADIF));		//Wait for ADC output registers to be loaded
	cli();   
	result = ADCW;							//Get 10 bit value
	sei();
	return (uint8_t)(result >> 2)	;			// Return 8 bit result
		

}

//---------------------------------------------------------------------------
/* Compute centroid for the 6 channel active IR object detector.
 
*/

int getIRcentroid()
{
	uint8_t i ;
	unsigned int low, sumA, sumB;
	int centroid;
	
	low = 255;
	for( i=0;i<6;i++){		//Find the least bright reading  and store value in "low"
		if(activeIR.data[i] < low) low = activeIR.data[i];
	}
	
	sumA = sumB = 0;
	for( i=0;i<6;i++){				//Compute sums
		sumB += (activeIR.data[i] - low) * (i + 1);
		sumA += activeIR.data[i] - low;
	}
	
	centroid = (sumB * 16) / (sumA/2) ; //Compute and scale centroid  32 * (sumB/sumA)
	centroid = centroid - 112;			//Make it zero centered.
	
	return centroid;
	
	
	
}


//----------------------------------------------------------------------------

/* Read 6 channels of active IR to detect objects in the path.
 
   6 IR LEDs are turned on in sequence and the reflected light from each
	is measured with the photodiodes and amplifier.
    A photodiode reading is taken first without any IR LEDs turned on to
   get a zero reference.  This bias is subtracted from all 6 readings.
 
  Also, this finds the channel with maximum brightness, the centroid, and contrast.
  
 */

void refreshIRdata()
{
	int i,x;
	uint8_t max = 0;
	
	uint8_t bias = get_adc8(AD_IR_PULSE);  //Get IR photodiode level for ambiant light (bias)
	
	for(i=0;i<6;i++){
		x = readActiveIR(i) - bias;  //Read all 6 active IR sensors and subtract bias
		if(x < 0) x = 0;
		activeIR.data[i] = (uint8_t)x;
		
	}
	
	for(i=0;i<6;i++){
		if(activeIR.data[i] > max) max = activeIR.data[i]; //Find max brightness for object sensors
	}
	
	activeIR.brightness = max;
	activeIR.centroid = getIRcentroid();
	activeIR.contrast = getContrast(6,activeIR.data,&activeIR.mean);
	activeIR.map = getMap(6,activeIR.data,activeIR.mean);
	
		
	
	
	
}

//----------------------------------------------------------------------------

//Get table edge detector values
//Note that due to Op-AMP saturation issues on very bright surfaces I had to put 100us delays between
//successive readings to allow the OP-AMP to recover fully.

void refreshEdgeSensors()
{

	int bias,x;
	
	
	//Be sure we wait 100us for photodiode recovery since last reading.
	get_adc8(AD_EDGE_PULSE);			//Dummy AD converter reads to waste 100us
	get_adc8(AD_EDGE_PULSE);
	
	
	bias = get_adc8(AD_EDGE_PULSE);  //Get IR photodiode level for ambiant light
	x = readActiveIR(6) - bias;			//Get left edge detector
	if(x < 0) x = 0;
	edge_left = (uint8_t)x;
	
	get_adc8(AD_EDGE_PULSE);			//Dummy AD converter reads to waste 100us
	get_adc8(AD_EDGE_PULSE);			//We need 100 us recovery time for the photodiode and amplifier
										//between readings
	
	x = readActiveIR(7) - bias;			//Get right edge detector	
	if(x < 0) x = 0;
	edge_right = (uint8_t)x;
		
	
}

//---------------------------------------------------------------------------

//Read 8 line tracking CDS photocells into line.data[] array.

//White levels for each cell are corrected using values stored in the EEPROM.
//See "saveLineWhiteLevels()" for how they were stored there.
//Note: Lower values equal brighter light.

void readLinePhotocells()
{
	uint8_t adr;
	int x;
	
	eeprom_read_block((void*)&line.data, (const void*)0, 9);  //Read back stored white level references
	
	if(checksum(line.data,8) != line.data[8]) {
		eeprom_read_block((void*)&line.data, (const void*)0, 9); //Read spare values if main corrupted
																//Need to eventually add code to check the spares too.
		}
	
	for(adr=0;adr<8;adr++){
		if(adr & 1) sbi(PORTC,0); else cbi(PORTC,0);
		if(adr & 2) sbi(PORTC,1); else cbi(PORTC,1);
		if(adr & 4) sbi(PORTC,2); else cbi(PORTC,2);
		
		_delay_us(20);							//Wait about 20uS for analog data to settle
		
		x = (int)get_adc8(AD_LINE_FOLLOW) - (int)line.data[adr]; //Get data and subtract white level bias
		if(x < 0) x = 0;									//Do not allow negative values
		
		
		line.data[adr] = (uint8_t)(x & 255);  //store corrected value
		
		
	}
	
}

//-------------------------------------------------------------------------
//Compute line photocell centroid.  Assumes 8 values of brightness are in line.data[] array.
// Note: Lower values equal brighter light.
//
int getLineCentroid()
{   uint8_t i ;
	unsigned int low, sumA, sumB;
	int centroid;
	
	low = 255;
	for( i=0;i<8;i++){		//Find the brightest photocell and store value in "low"
		if(line.data[i] < low) low = line.data[i];
	}
	
	sumA = sumB = 0;
	for( i=0;i<8;i++){				//Compute sums  (relative to the "low")
		sumB += (line.data[i] - low) * (i + 1);
		sumA += line.data[i] - low;
	}
	
	centroid = (sumB * 16) / (sumA >> 1) ; //Compute and scale centroid  32*(sumB/sumA)
	centroid = centroid - 144;				//Make it zero centered
	line.centroid  = centroid;
	
	return centroid;
}


//-----------------------------------------------------------------------------
		
//-----------------------------------------------------------------------------
//Cpompute and return contrast and mean.
int getContrast(uint8_t n, uint8_t *data, uint8_t *mean )
{
	uint8_t high,low,i;
	unsigned int contrast;
	high = 0;
	low = 255;
	
	for(i=0;i<n;i++){
		if(data[i] > high) high = data[i];
		if(data[i] < low) low = data[i];
	}
	
	uint8_t sliceLevel = high - ((high-low) / 2); //Find level half way between high and low
	contrast = high - low;
		
	*mean = sliceLevel;
	
	return contrast;
	
}
//---------------------------------------------------------------------------


uint8_t getMap(uint8_t n, uint8_t* data, uint8_t mean)    //Photocells reading over the mean value 
						//are black and the assgned bit in "map" is set.
{						//Left most photocell is bit 7, right most is bit 0.
	uint8_t map = 0;
	uint8_t i;
	
	for(i=0;i<n;i++){
		map = map >> 1;
		if(data[i] > mean) map = map | 0x80;
	}
	
	
	return map;
	
}

//---------------------------------------------------------------------------------

/* Get line photocell data and process it in various ways
 */
	
void refreshLineData()
{   int i, b;
	uint8_t c, threshold;
	readLinePhotocells();
	line.contrast = getContrast(8,line.data,&line.mean);	
	line.map = getMap(8,line.data,line.mean);
	
	line.centroid = getLineCentroid();
	
	if(line.present == TRUE) threshold = 9; else threshold = 11;  //provide some hystereses
	
	line.present = FALSE;
	for(i=0;i < 8;i++){
		if(line.data[i] >= threshold) line.present = TRUE;
		}
		
	if(line.present == FALSE) {
		line.map = 0;
		line.centroid = 0;
		}
		
	line.width = 0;
	c = 1;
	for(i=0;i<8;i++){
		if(line.map & c ) line.width++;		//count the 1's in line.map to get width
		c = c << 1;		 
		
	}
		
	
	b = 0;
	for(i=0;i<8;i++) b += line.data[i];  //Sum all the photocells
	b = b / 8;  //get average
	line.brightness = 255 - b; //Invert to make brightness a positive function, 0-255

	
}

//------------------------------------------------------------------------------

//Computer centroid for the 2 beacon photocells

int getBeaconCentroid()
{   
	unsigned int  sumA, sumB;
	int centroid;
	
	
	//Compute sums
	sumB = beacon.left + (beacon.right  * 2);    //Max = 255 + (255*2) = 755
	sumA = beacon.left + beacon.right;			//Max = 255+255= 510
	
	
	centroid = (sumB * 80)  / sumA ; //Compute and scale centroid. Keep (sumB * X) under 0xffff.
	centroid = centroid - 120;
		
	return centroid;
}

//------------------------------------------------------------------------------

//Read the left and right beacon photocells
//Compute brightness and get centroid.

void getBeacon()
{
	
beacon.left = 255 - get_adc8(AD_BEACON_LEFT);
beacon.right = 255 - get_adc8(AD_BEACON_RIGHT);
beacon.brightness = (beacon.right + beacon.left) / 2;
beacon.centroid = getBeaconCentroid();

	
}
	

//----------------------------------------------------------------------

/* Adjust motor speeds.  */

#define CURRENT_LIMIT 44

/* Current limiter:
   1300 ohm motor current load resistor gives .5 volts per amp.
   25 AD converter units per amp.
   Value of 44 is 1.75 amps (sum of both motors)

*/

/* Locked Antiphase PWM Motor Speed Control at 28.8 KHZ.
   Note that a 50% duty cycle PWM waveform equals ZERO speed.
   Duty cycles above or below 50% run the motors either forward or reverse.
   eg: 75% is 1/2 speed forward while 25% is 1/2 speed reverse.
   
   Speed is set independently for left and right motors with l_speed and r_speed variables.
   The Range is -126 to +125 with zero being stop.   
   100% duty cycle is not allowed because the H-Bridge has bootstrap capacitors which require pulses
   at all times to work correctly.
   
*/
void set_motor_speed(int l_speed, int r_speed){
      uint8_t hbridge_enable;
      
      int spd;

      uint8_t lm_pwm, rm_pwm;				
     
      if((l_speed != 0) || (r_speed != 0)) timer_mstop = 40;  //400ms breaking when speed goes to zero
     
      /* Right motor PWM control */
       
       spd = r_speed ;  //change sign to get the direction right instead of swapping motor wires if required

       spd = hard_limit_range(spd,125,-126);  //Limit range from +125 to -126 so h-bridge bootstrap always has pulses
       rm_pwm = (uint8_t)spd + 128;
       OCR1BH = 0;                      //set right PWM
       OCR1BL = rm_pwm;


       /* Left motor PWM control */   
       
       spd = l_speed;

       spd = hard_limit_range(spd,125,-126);
       lm_pwm = (uint8_t)spd + 128; 
       OCR1AH = 0;                      //set left PWM
       OCR1AL = lm_pwm;

       /* H-Bridge enable control */

       hbridge_enable = 1;      //Assume h-bridge will be enabled
                                //Then look for reasons to disable it below...

         if (m_current > CURRENT_LIMIT){
              hbridge_enable = 0;  // Limit drive motor current to 1.75 AMPs (sum of both motors)
         }

         if ((l_speed == 0)
             && (r_speed == 0)
             && (timer_mstop == 0)) hbridge_enable = 0;   //Disable H-Bridge if bot is stopped
	
	if(hbridge_enable == 0) cbi(PORTB,MOT_ENAB); else sbi(PORTB,MOT_ENAB);       

 }


//-----------------------------------------------------------------------------



   /* Send debug data to LCD display*/
  
void writelcd(char *cp){
      
	while(txstate != 0);	//Wait until UART not busy      
	if(strlen(cp) < TXBLEN) strcpy(tx_buf,cp); else return;  //Copy string to tx buffer
	
	txstate = 4;         	//Tell scheduler it's ready to transmit
	sbi(UCSR0B, TXCIE0); 	//Enable TX UART interrupts     
	cbi(UCSR0A,TXC0);		//Set transmit complete bit
	UDR0 = 0x12;			//  cursor to home 
   
      
   }
   
//-------------------
void clearScreen()
{   char s[] = {0x0c, 0};
    writelcd(s);
}
    
    
//----------------------------------------------------------------------
   void refreshDebug()
   {	int i,j;
   
   
   for(i=0,j=0;i<6;i++){
	   bin2hex(sbuf+j,debug[i]);  //Convert all 6 debug values to ascii hex in the sbuf array
	   sbuf[j+2] = ' ';
	   j = j+3;
           if(i == 2) {        //Next line on LCD display after 8 chars.
             sbuf[j++] = '\r';
             sbuf[j++] = '\n';
           }
   }
   writelcd(sbuf);
   }


//----------------------------------------------------------------------

char debugReady()
{
	if(txstate != 0) return 0;
	else return 1;
}
	
//----------------------------------------------------------------------

// Wait t*10ms and display time remaining on LCD display

void delay_start(int t, PGM_P s){
	int tenths;
	
	timer10ms = t;	
		
	while(timer10ms > 0 ){		//Wait here for t*10ms  and show time remaining on LCD
	  
	  tenths = timer10ms / 10;   //Converts hundredths to tenths.
	  
	  if(txstate == 0){
		itoa(tenths,sbuf,10);		//Convert integer to decimal string
		if(tenths > 9){
				sbuf[2] = sbuf[1];
				sbuf[1] = '.';		//Insert decimal point
							
			}
			else
			{
				sbuf[2] = sbuf[0];
				sbuf[1] = '.';
				sbuf[0] = '0';		//Insert leading zero and decimal point
				
			}
		 
		  sbuf[3] = '\r';
		  sbuf[4] = '\n';
		  strcpy_P(&sbuf[5], s);  //Copy Title string from Program memory to ram buffer
		  
		  writelcd(sbuf);			//Write to LCD display
		
		}
	}
	  
  }


//----------------------------------------------------------------------


/* UART Transmit interrupt handler state machine */ 

SIGNAL(USART0_TX_vect )
{ 
   

      switch(txstate) {
      case 0:  	break;
      case 1:	//unused
      case 2:	//unused
      case 3: 	//unused
      case 4: 	tx_ptr = 0;
         	tx_ctr = strlen(tx_buf); 
         	txstate = 5;
		//fall through...

      case 5:  	UDR0 = tx_buf[tx_ptr++] ;  //Now send buffer contents
            	tx_ctr--;
            	if (tx_ctr == 0){ 
               	   txstate = 6;
               
            	}
         	
            break;

      case 6:  
				txstate = 0;
				cbi(UCSR0B,TXCIE0);		// Clear TX complete interrupt enable bit
				sbi(UCSR0A,TXC0);    //Set transmit complete bit
	
               break;



      }

      
}

//----------------------------------------------------------------------------

/* Come here 10,000 times per second.   
   Process various software timers.
   100us resolution.
 
 Note: Timer variables used here must be declared volatile.
 
 */
SIGNAL(TIMER2_COMPA_vect )
{

		
		
		rnd++ ;
		rnd = rnd ^ beacon.left;   //Make a random number
		
		if(timer100us > 0) timer100us--;		//General purpose 100us/tick timer
		if(timer_pid > 0) timer_pid--;			//100us per tick pid timer

		if(--prescale == 0){      //This executes 100 times per sec.
			prescale = 100;
         
			if(prescale1sec > 0) prescale1sec--;
			if(timer10ms > 0) timer10ms--;
			if(timer_trip > 0) timer_trip--;
			if(timer_mstop > 0) timer_mstop--;
			if(timer_overcurrent > 0) timer_overcurrent--;
			
			if(timer_move > 0) timer_move--;
			if(timer_delay > 0) timer_delay--;
                 
			}

      
		if(prescale1sec == 0){  //This executes once per second
			prescale1sec = 100;
			
			if(prescaleMinute > 0) prescaleMinute--;
			run_seconds++;
			
        
			}

		if(prescaleMinute == 0){ //This executes once per minute
			prescaleMinute = 60;
			minutes++;        
			}

		 
    /* End interrupt code */
    

}

//-------------------------------------------------------------------------

/* Quadrature encoder interrupt handlers */

/*  36.8  counts per inch  or 0.0272 inch per count  */

//LEFT side quadrature clock
SIGNAL(INT0_vect)
{
	if(PIND & 0x80) dir_left = FWD; else dir_left = REV;
	if(dir_left == FWD){
		odo_left++; 
		trip_odo_left++;
	}else {
		odo_left--;
		trip_odo_left--;
	}
	
}

//Right side Quadrature clock
SIGNAL(INT1_vect)
{
	if(PIND & 0x40) dir_right = FWD; else dir_right = REV;  
	if(dir_right == FWD){
		odo_right++; 
		trip_odo_right++;
	}else {
		odo_right--;
		trip_odo_right--;
	}
	
	
}


//------------------------------------------------------------------------

//Read pot and motor current values.
//Also beacon values.

void refresh_sensors(void)
{

 	pot = get_adc8(AD_POT);	
	m_current = get_adc8(AD_MOTOR_CURRENT);
	
	getBeacon();
	
	
		
}	
	


//--------------------------------------------------------------------------

//Returns true if motors have been pulling excessive current for too long
//Unused in this applicaton
/*
uint8_t over_current(void)
{

	motor_current = get_adc8(AD_MOTOR_CURRENT);
	
	if (motor_current < CURRENT_LIMIT) timer_overcurrent = 60;  //Reset timer to 600ms if no overcurrent
	
	if ((timer_overcurrent == 0) && (motor_current >= CURRENT_LIMIT)) return 1;
	
	return 0;
	
	}

*/


//-------------------------------------------------------------------------
//Correlator
//Enter with most recent sensor value.
//Do 16 sample correlation with square wave and return correlation value.
//Sq wave has 8 units positive and 8 units negative.

int correlate(int s)
{   uint8_t i;
	int a;

	for(i=15;i > 0; i--) samples[i] = samples[i-1];  // shift old data by 1 place in 16 integer array
	samples[0] = s;									// put in new data
	a = 0;											// Clear accumulator
	for(i=0;i<8;i++) a = a - samples[i];			// sum the negative half
	for(i=0;i<8;i++) a = a + samples[i+8];			// sum the positive half
	
	return a / 16;									// return correlation value divided by 16
	
}
	

//-------------------------------------------------------------------------
//Rotate in place 360 degrees to collect light readings.
//Then rotate to the angle most likely to be facing the beacon.
//

int face_beacon(char test)
{  uint8_t i;
   int deltaLR,  maxc, lastd, c;
   long odo_mark_beacon, odo_next_sample;
	maxc = 0;
     						

	if(test == TRUE) delay_start(500,facebeacon);
	
	timer10ms  = 200;
	refresh_sensors();
	speed = 70;    //pot - 128; //Use pot for testing
	set_motor_speed(-speed, speed);  //Spin CCW
  
	getBeacon();
	beacon.minbright = 255;
	beacon.maxbright = 0;
	deltaLR = beacon.left - beacon.right;
	lastd = deltaLR;
	timer100us = 14;
	
	for(i=0;i<16;i++) samples[i] = 0;  // Clear correlator
	
	trip_odo_left = trip_odo_right = 0;
	odo_mark_beacon = 0;
	odo_next_sample = 0;   
	  
	while((trip_odo_right < 910)){  // Rotate for 360 degrees 
  
	if(trip_odo_right >= odo_next_sample ){	//Take light readings every 4 degrees
		odo_next_sample = trip_odo_right + 13 ;   // aprox. 5 degrees rotation per sample		
		getBeacon();		//get latest beacon light readings
		deltaLR = beacon.left - beacon.right;  //compute right-left difference
		c = correlate(deltaLR);
		
		if(c > maxc){
				maxc = c;
				odo_mark_beacon = trip_odo_right - 100;  //Subtract 40 degrees of correlator delay
				}
					
		if(beacon.brightness < beacon.minbright) beacon.minbright = beacon.brightness;  //Get min and max brightness levels
		if(beacon.brightness > beacon.maxbright) beacon.maxbright = beacon.brightness;
		
	}
  }

	//calculate the shortest way to get back facing the beacon, CW or CCW.
	if((trip_odo_right - odo_mark_beacon)	< 455){
		set_motor_speed(speed, -speed);	
		while((trip_odo_right-50) > odo_mark_beacon);  //rotate CW back to point at beacon. -50 compensates for overshoot
		}
		else{
		    set_motor_speed(-speed,speed);			  
			while((trip_odo_right+50) < (910+odo_mark_beacon));  //Rotate CCW to face beacon. +50 compensates for overshoot
			}
		
	set_motor_speed(0,0);
	
	if(test == TRUE){
		delay(15);
		
	debug[0] = maxc & 255;
	debug[1] = trip_odo_right >> 8;
	debug[2] = trip_odo_right & 255;
	debug[3] = beacon.maxbright;
	debug[4] = odo_mark_beacon >> 8;
	debug[5] = odo_mark_beacon & 255;
	refreshDebug();
	while(1 == 1);    //Stall here forever if in test mode.
	}
  
  
			
	return maxc;

}
//------------------------------------------------------------------------------

// Update PID control value.  
// Input is latest error term "p" and pointer to PID data structure "*pid" .



int computePID(int p, struct PID *pid)
{
	  
	int il = 0;
	
		
	if(timer_pid == 0){
		timer_pid = pid->t * 10;	 // t is given in 1ms increments, timer is 0.1 ms per tick
		
		pid->d = (p - pid->p) * pid->kd ;		//compute differencial d * kd
		pid->d = pid->d / pid->t;			//Divide by delta t
		
		pid->p = p;
		
		il = hard_limit_range(p,50,-50);  //Limit integrator drive level to reduce windup
		il = il * pid->t;					//mpy by delta t
		pid->i = (il + pid->i);				//compute integral i 		
		il = pid->i / pid->ki;				//Divide by scaling factor ki
		il = hard_limit_range(il,pid->ilim,-pid->ilim);  //Limit max range of integral
		
		pid->u = (pid->p * pid->kp) + pid->d + il;
			
	}
	
	return pid->u;
}

//------------------------------------------------------------------------------

//This rotates the bot CW or CCW at an intersection to change course.
//Used to turn left or right in the maze
//PID feedback control is used to quickly center on the new line.

void rotateToLine(int spd, uint8_t dir)
{
	int   u;
		
	refreshLineData();
	if( dir == CCW) set_motor_speed(-spd,spd);
		else set_motor_speed(spd,-spd);
	while(line.present == TRUE) refreshLineData();;  //Rotate off the current line if present
	delay(1);										 //Wait to be sure we are off the previous line
	while(line.present == FALSE) refreshLineData();;	//Continue until we see a new line
	
	timer10ms = 20;					//Allow 200ms to settle on line
	
	//Now use PID to center on the line.
	while(timer10ms > 0){
		refreshLineData();			
		u = computePID(line.centroid,&pid_line);		
		u = hard_limit_range(u,spd, -spd);
		set_motor_speed(-u,u);					//servo  to line center
		
	}
	
	set_motor_speed(0,0);						//Stop.
	
	
}


//------------------------------------------------------------------------------

void odometer_debug()
{
	if(debugReady()){
	debug[0] = (trip_odo_right >> 16) & 255;
	debug[1] = (trip_odo_right >> 8) & 255;
	debug[2] = (trip_odo_right & 255);
	debug[3] = (trip_odo_left >> 16) & 255;
	debug[4] = (trip_odo_left >> 8) & 255;
	debug[5] = (trip_odo_left & 255);
	refreshDebug();
    }
}
	
//------------------------------------------------------------------------------
void driveStraight(int distance, int speed)
{   long dcnt;
	long d;
	int spd;
	trip_odo_left = trip_odo_right = 0;
	dcnt  = CPI * distance;   //d now has odometer counts required
	spd = speed;
	while(trip_odo_left < dcnt){
		if((dcnt - trip_odo_left) < 200) spd = speed / 2;
		
		refresh_sensors();
		
		d = trip_odo_right - trip_odo_left;
		if(d > 5) d = 5;    //Limit odometer difference range to +/- 5
		if(d < -5) d = -5;
		
				
		l_speed = spd + (d * 8);    //Correct steering based on odometer count difference.
		r_speed = spd - (d * 8);
		set_motor_speed(l_speed, r_speed);
		
		odometer_debug();
		 
	}
	
	l_speed = r_speed = 0;
	set_motor_speed(l_speed, r_speed);
		
		
	
}

//----------------------------------------------------------------------------
//Wheels are 7.875 inches apart.
//To turn 360 degrees requires they each travel the circumference which is pi*d = 24.74 inches.
//with 36.9 quadrature counts per inch they need to travel 36.9 * 24.74 = 912.9 counts
// or 2.536 counts per degree.
#define C360 = 913
void rotate(int degrees, int speed, char dir, char option)
{   
	
	long dcnt;
	
	int spd, stable,last;
	int ss;
	
	
	dcnt = (degrees * 2) + ((degrees * 53) / 100);   //
	
	stable = 0;	
	last = 0;
	spd = speed;
	
	if(dir == CW){				//Set direction to rotate  CW or CCW
		l_speed = spd;
		r_speed = -spd;
		trip_odo_left = -dcnt;
	    trip_odo_right = dcnt;
		} else {
			l_speed = -spd;
			r_speed = spd;
			trip_odo_left =  dcnt;
			trip_odo_right = -dcnt;
			
			}
		
		timer10ms = degrees * 2; //(degrees * 3) / 4;		//Allow 20ms time per degree for turn
		if(degrees < 30) timer10ms = 30;  //300ms minimum
		
		while(timer10ms > 0){    //Allow max time for this to happen so it won't get stuck ... ever.
		    
			refresh_sensors();	
				
			if(option == 1){
			    
				refreshIRdata();
				if((activeIR.brightness > 0x30) && (abs(activeIR.centroid) < 10)){
					timer10ms = 0;  //Quit now if object in front of bot.
					}
			}
			
							
			
			//Now let the trip odometer error term totally control the motors like a servo.
			//Eventually the trip odomenter errors will become zero and the motors will stop.
			
			 l_speed = -trip_odo_left * 3;
			l_speed = hard_limit_range(l_speed,spd,-spd);
			 r_speed = -trip_odo_right * 3;
			r_speed = hard_limit_range(r_speed,spd,-spd);
			
						
			if(timer100us == 0){
				timer100us = 100;    //10ms
				ss = abs(trip_odo_right) + abs(trip_odo_left);				
				if(ss != last) stable = 0; else stable++;	//It's been stable for this many 10ms intervals				
				if(stable > 3) timer10ms = 0;  //Exit if trip odometer stable for 30ms
				last = ss;
				
			}
			
			
			set_motor_speed(l_speed, r_speed);	
			
			if(option == 2){
					if((abs(r_speed) < 20) && (abs(l_speed) < 20)) timer10ms = 0;
					}
							
			
		}
		
		//Finished turning
		l_speed = r_speed = 0;				//Set motor speeds to zero
		set_motor_speed(l_speed, r_speed);
	
		
		
	
	
		
}

//-----------------------------------------------------------------------------

/* DEAD RECKONING  Switch position = 2
 Rear skid is used as a reference and must be on the start dot.
 
 1. Drive 43 inches to first point.
 2. Turn 120 degrees clockwise
 3. Drive 48 inches to second point.
 4. Trun 120 degrees clockwise.
 5. Drive 53 inches to original start point.
 
 */

void deadReckoning()
{   
	delay_start(500,deadrec);  // 5 sec delay
	refresh_sensors();
	int p = 45; //pot - 128;
	driveStraight(43, p);
	rotate(120,p,CW,0);
	driveStraight(48,p);
	rotate(120,p,CW,0);
	driveStraight(53,p);
	
	
	
	
}



//-----------------------------------------------------------------------------

/* BEACON KILLER   Switch position = 1
 
 1. Spin around 360 degrees looking for beacon and remember which direction.
 2. Rotate back to point at beacon and start running forward.  (uses quadrature encoders)
 3. Steer towards beacon light unless obstacle is seen.
 4. Steer away from obstacle until it is no longer seen.
 5. When light intensity reaches a certain level turn off the obstacle avoid logic.  ( < 2 feet from beacon)
 6. When obstacle avoidance is off and the light intensity drops below a threshold assume beacon 
	off switch was hit and stop.
 
 */


int beaconFinder()
{
	int speed, c, x;
	long distance;
	uint8_t near,stop;
	
	delay_start(500,beaconfinder);
	
	x = face_beacon(0);	
	if(x < 0x20) return -1;   //No beacon found
	
	trip_odo_right = trip_odo_left = 0;
	
	
	near = FALSE;
	stop = FALSE;
	
	while(stop == FALSE){
		refresh_sensors();
		refreshIRdata();	
		speed = pot - 128;
		
		c = beacon.centroid * 2;   //Get beacon heading
		
		//Drive towards the beacon
		l_speed = speed + c;
		r_speed = speed - c;
		
		
		//Compute active IR obstacle steering
		c = activeIR.centroid ;
		
		if(c < 0){			//Split centroid into left and right "avoid" functions 	 
			c += 0x50; 
			if(c < 0) c = 0;
		}
		else{
			c -= 0x50;
			if(c > 0) c = 0;
		}
		
		c = c * 2;
		
		//Subsume motor control from beacon steering if obstacle present...
		//... and drive around obstacles.
		if((activeIR.brightness > 30) && (near == FALSE)) {  
						
			l_speed = speed - c;		//Use "avoid" function
			r_speed = speed + c;
						
		}
		
		
		set_motor_speed(l_speed,r_speed);   //Set motor speeds to computed values, either towards beacon or away from obsticals.
		
		
		distance = (trip_odo_left + trip_odo_right) / (2 * CPI);  //Distance traveled in inches.  
														//May use this someday to determine when to disable object ovoidance
														//instead of beacon brightness.
				
		if(beacon.brightness > 0xb8)  near = TRUE;
		
		if((beacon.brightness < 0x50) && (near == TRUE)) stop = TRUE;
		
		
		
	}
	
	
	
	return 0;
	
}
	
	
	
//-----------------------------------------------------------------------------

/* LINE FOLLOWER, basic and advanced,  Switch = 0 
 
 1.  Use PID servo loop to follow 3/4 inch black line on white background.
     Zero radius turns are handled as a special case and are detected with the
	 IR edge detector sensors.  Left or right zero radius turns are "remembered"
	 when the sensors cross the line and the bot will stop and 
	 rotate CW or CCW to re-aquire the line.
 
	The PID parameters are stored in the "pid_line" struct and initialized 
	in main when the program starts.
 
	Speed is set with the pot.
 
 */

void lineFollow()
{   int u;	
	uint8_t mem_flag = LEFT;
	
	delay_start(500,linefoll);  //5 sec delay
	run_seconds = 0;
		
	while(run_seconds < 90){   //This loop repeats about 1100 times/second
		
		refreshLineData();		
		refresh_sensors();
		 speed = pot - 128; 
		
		refreshEdgeSensors();				   //Use edge sensors to detect zero radius turns
		if(edge_left < ET)  mem_flag = LEFT;   //   Remember line was on the left ...
		if(edge_right < ET)  mem_flag = RIGHT;  //  ... or the right in case we lose it on zero radius turn.
		
		if(line.present == TRUE){  //If we see a line, track it		
		
			u = computePID(line.centroid,&pid_line);  //Update the PID feedback loop.
			r_speed = speed + u;					  //Set motor speed values.
			l_speed = speed - u;	
				
		 } 
		else{  //If we don't see a line, stop and spin to re-aquire line
			   //This is required for zero radius corners
			
			pid_line.i = 0;  //Zero the integral because we need to reaquire the line
			
			switch(mem_flag){
					
				case LEFT:	l_speed = -speed;  // CCW
							r_speed = speed;
							break;
					
				case RIGHT:	l_speed = speed;  // CW
							r_speed = -speed;
							break;
			
			}
		}
		
		
		if(line.brightness < 0xd0){     //Go straight if over a intersecting black line
			l_speed = r_speed = speed;
		}
		
		set_motor_speed(l_speed, r_speed);  //Now apply the speed values to the motors .
		
		
		
		/*
		if(debugReady()){			//DEBUG stuff
			debug[0] = OCR1BL;
			debug[1] = OCR1AL;
			debug[2] = mem_flag;
			debug[3] = line.contrast;
			debug[4] =  line.centroid & 255;
			debug[5] =  line.map;
			refreshDebug();
			
		} */

	}
}



//---------------------------------------------------------------------------

/* BULLDOZER   Switch = 3
 
 1. Start with front edge sensors hanging off the table facing away from objects.
 2. Measure edge sensor values for floor.
 3. Backup 6 inches and measure the edge sensor values for the table.
 4. Compute a threshold between floor and table.
 5. Rotate CW 113 degrees and go until an edge or object is detected.
    If edge detected rotate CW 160 degrees.  Continue to zig zag until object is seen.
 6. When object is seen using active IR, steer towards it and run until an edge is detected.
 7. Do a backup, rotate, forward, backup until both edge sensors are off the table.  
    Object should be pushed off.
 8. Back up, rotate, continue zig zag pattern while looking for objects.
 
 Note that bot can get lost after steering towards objects and 
 start zig zagging the long table dimension but it still works.

*/
void bulldozer()
{   int sliceL, sliceR, floorL, floorR, tableL, tableR, c ;
    uint8_t  d , deg, object;
	uint8_t edge, expected_edge, edge_status;
	delay_start(500,bulldoz);  //5 sec delay
	
	refresh_sensors();		
	speed = pot - 128;
	
	run_seconds = 0;
	edge = NONE;			//Neither edge detector triggered yet.
	expected_edge = LEFT;	//We expect the left edge first.
	edge_status = NONE;
	object = FALSE;
	
	refreshEdgeSensors();
	floorL = edge_left  ;  //get light level looking at the floor
	floorR = edge_right   ;
	
	
	trip_odo_left = CPI * 6;   //Setup to back up 6 inches
	while(trip_odo_left > 0) set_motor_speed(-60,-60);
	set_motor_speed(0,0);
	delay(20);
	refreshEdgeSensors();
	tableL =  edge_left  ;  //get average table light level
	tableR =  edge_right  ;
	sliceL = floorL + ((tableL - floorL) / 5) ;   //compute slice value 20% up from dark
	sliceR = floorR + (( tableR - floorR) / 5) ;
	
	d = CW;
	deg = 113;
	rotate(deg,speed,d,0);		//Rotate 113 deg
		
	
	
	while(run_seconds < 90){  //Limited to 90 seconds
		refresh_sensors();		
		speed = pot - 128;
		
		
		refreshIRdata();
		
		c = activeIR.centroid ;   //Get object heading
		
		if(activeIR.brightness < 0x90)  //Really bright object means we probably captured it
				object = FALSE;
				else object = TRUE;
				
		
		if(activeIR.brightness < 0x20) c = 0;  //Ignore far off objects, they may be hallucinations or circuit noise.
		l_speed = speed - c;	//Drive towards the object
		r_speed = speed + c;
		
		refreshEdgeSensors();
		if((edge_right < sliceR) && (edge_left < sliceL)) edge_status = BOTH;
		if((edge_right < sliceR) && (edge_left >= sliceL)) edge_status = RIGHT;
		if((edge_right >= sliceR) && (edge_left < sliceL)) edge_status = LEFT;
		if((edge_right >= sliceR) && (edge_left >= sliceL)) edge_status = NONE;
		
		
		if(edge_status == BOTH){   //Edge detected with both left and right sensors
		    
			set_motor_speed(-speed,-speed);
			trip_odo_left = CPI * 4;
			while(trip_odo_left > 0) ; //Reverse 4 inches
			
			
			set_motor_speed(0,0);		
			
			switch( edge){
			case NONE:	deg = 180, d = CCW ; break;
			case LEFT:	deg = 160, d = CW; break;
			case RIGHT:	deg = 160, d = CCW; break;
			}
			
			
			rotate(deg,speed,d,1);  //Now make the turn
				
			
			} 
						
			if((edge_status == RIGHT) && (object == FALSE)){
				set_motor_speed(-speed,-speed);
				trip_odo_left = CPI * 4;
				while(trip_odo_left > 0) ; //Reverse 4 inches
				
				rotate(140,speed,CCW,1);
				edge = edge_status = NONE;
				expected_edge = LEFT;
				}
				
			if((edge_status == LEFT) && (object == FALSE)){
				set_motor_speed(-speed,-speed);
				trip_odo_left = CPI * 4;
				while(trip_odo_left > 0) ; //Reverse 4 inches
				
				rotate(140,speed,CW,1);
				edge = edge_status = NONE;
				expected_edge = RIGHT;
				}
				
				
			
		   //Edge detected with either edge sensor (Exclusive OR)
		   //This causes the bot to square itself with the edge to ensure the object is pushed off
		   
			if((edge_status == RIGHT) && (object == TRUE)){  
				r_speed = -40; 
				l_speed = 40;  
				
				edge = edge_status;  //Detect which side went off first
				
			}
			
			if((edge_status == LEFT ) && (object == TRUE)) { 
				l_speed = -40;	
				r_speed = 40;  
				 
				edge = edge_status;  //Detect which side went off first
				
			}
			
			
		
				
		set_motor_speed(l_speed,r_speed);   //Set motor speeds to computed values		
			
		
		}
	
	
	 
	
	
}

//----------------------------------------------------------------------------

/* Switch Position 4:  Line maze.

 Right hand follow rule.  No learning in this version.
 1. If dead end, turn around 180 degrees
 2. If choice is left or straight, go straight
 3. If choice is right or straight, go right.
 4. If choice is left or right, go right.
 
 5. A fat line indicates we are done.
 
 Reverse left and right for the left hand rule.
 
*/

void lineMaze(char rule)
{
    char noLine;
	int u, d, n, q, deg, last_centroid;
	
	long dest;
	
	char left_flag, right_flag;
	char turn = CW;
	char fat_line_counter = 0;
	
	uint8_t tst = 255;
	uint8_t tst2 = 0;
	unsigned int tst3 = 0;
	
	
	delay_start(500,linemaze);  //5 sec delay
	run_seconds = 0;
	n = 0;
	q = 0;
	d = 0;
	last_centroid = 0;
	noLine = 0;
	
		
	left_flag = right_flag = FALSE;
	timer_trip = 3 ; //30ms
	
	
	
	//Maze loop
	while(run_seconds < 90){  //Run for 90 seconds max.
		refreshLineData();
		refresh_sensors();
		refreshEdgeSensors();				//Use edge sensors to detect intersections
		
		
		speed = 100 ; //Set speed		
						
		if((line.width > 0) && (line.width < 3)){  //If we see a normal thin line, track it
					
		u = computePID(line.centroid,&pid_line);  //Update PID feedback loop
				
		r_speed = speed + u;		//Apply updated error term
		l_speed = speed - u;
				
		} else {l_speed = speed; r_speed = speed;}  //else fat line. Assume 90 deg corner or intersection. Don't track.
		
			
		/* Intersection detecton and Left or right turn process */
		
		
		if((edge_left < ET) || (edge_right < ET) || (line.present == FALSE)){
			set_motor_speed(60,60);				//Slow down a little and proceed straight.
			left_flag = right_flag = FALSE;		//reset intersection flags	
			dest = trip_odo_left + (CPI * 2);  //Move 2 inches past intersection			
			while(trip_odo_left < dest) {
				if(edge_left < ET) left_flag = TRUE;	//update intersection flags as we move	
				if(edge_right < ET) right_flag = TRUE;
				if(edge_left < tst) tst = edge_left;
				if(edge_left > tst2) tst2 = edge_left;
				if(edge_left == 0) tst3++;
			    refresh_sensors();
				refreshLineData();					//Update all the sensor data
				refreshEdgeSensors();
				
				
				} 
			
			/*
			set_motor_speed(0,0);    //Debug code commented out
			debug[0] = left_flag;
			debug[1] = right_flag;
			debug[3] = tst;
			debug[4] = tst2;
			refreshDebug();
			while(1 == 1);
			*/
					
				deg = 90;   //Assume 90 degree turn
				turn = NONE; //Default to straight ahead
			
				if(line.present == FALSE){
					if((left_flag == TRUE) && (right_flag == FALSE)) turn = CCW;  //Left turn corner	
					if((left_flag == FALSE) && (right_flag == TRUE)) turn = CW;  //Right turn corner
					
					if((left_flag == TRUE) && (right_flag == TRUE)){ // T intersection choice...
						if(rule == RIGHT) turn = CW;   //right
						if(rule == LEFT) turn = CCW;	//left
						}
						
					if((right_flag == FALSE) && (left_flag == FALSE)){  //Dead end, turn around
						turn = CW;
						deg = 180;
					}						
				}	
				
				if(line.present == TRUE){
					if((rule == RIGHT) && (right_flag == TRUE)) turn = CW;
					if((rule == LEFT) && (left_flag == TRUE)) turn = CCW;
													
				}
									
				
						
			if(turn != NONE) rotateToLine(speed,turn); //rotate(deg,speed,turn,0);		 // Now do the turn
				/*	
				set_motor_speed(0,0);  //Debug code
				debug[0] = deg;
				debug[3] = turn;
				refreshDebug();
				while(1 == 1);
			*/
				
			left_flag = FALSE;
			right_flag = FALSE;
			fat_line_counter = 0;   //Required to prevent false stops	
			noLine = 0;		
			l_speed = r_speed = speed;
			
			}
			
			
		/* End square detection */	
		if(timer_trip == 0){   //Every 20ms check line width to detect 4x4 inch end square
			timer_trip = 2;
			if(line.width > 3) fat_line_counter++;  // Count consecutive times we see a fat line.
				else fat_line_counter = 0;
			if(fat_line_counter > 4) run_seconds = 90;  // Yes, stop when we run into the 4x4 inch square
			
			if(line.present == FALSE) noLine++; else noLine = 0;
			
			} 
			
			
		set_motor_speed(l_speed, r_speed);  // set the motor speeds.
		
					
		

	}
}





		

//---------------------------------------------------------------------------

/* Switch Position F :  Debug Code to display Active IR object sensor values */

void test_sensors(void)
{	uint8_t adr = 0;

	unsigned int ctr,x;
	
		
	
	sbi(PORTC,3);		//Disable IR LEDs
	x = 0 ;
	
	while (1 == 1){
		
		irsensor[adr] = readActiveIR(adr);
		sbi(PORTC,3);
		_delay_ms(1);			//About 1 ms between pulses
		adr++;					//Next LED address
		x++;
		
		if (x >= 100){  //about 10 updates per sec
			for(ctr=0;ctr < 6;ctr++) debug[ctr] = irsensor[ctr];
			
			refreshDebug();
			x = 0;
			}
		}
		
		



}

//--------------------------------------------------------------------------

/* Active IR test readings.

Left and right edge sensors.
1. White paper		0x98
2. Brown table top  0x40
3. Off edge, beige carpet 0x00
4. Off edge, white paper  0x0a
5. Black tape, 3/4 inch   0x20


*/

//Switch = A
void test_ir()
{   int c;
	
	while(1 == 1){
		refreshIRdata();
		refreshEdgeSensors();
	
		debug[0] = activeIR.centroid;
		debug[1] = activeIR.map ;
		debug[2] = activeIR.brightness;
				
		c = activeIR.centroid ;
		if(c < 0) c += 0x50; else c -= 0x50;
		debug[3] = c & 255;
		
		debug[4] = edge_left;
		debug[5] = edge_right;

		
		delay(15);
		refreshDebug();
		
	}
	
	
}




//---------------------------------------------------------------------

//Switch position = B

void test_beacon()
{
	
	while(1 == 1){
		refresh_sensors();	
		
		debug[0] = beacon.left;
		debug[1] = beacon.right;
		debug[2] = beacon.brightness;
		debug[3] = beacon.centroid & 255;
		debug[4] = 0;
		debug[5] = 0;
		
		delay(10);
		if(debugReady()) refreshDebug();
	}
	
	
	
}



//Switch position = E
/*
void test_passive_sensors()
{

		
	while(1 == 1){
		refreshLineData();
		debug[0] = get_adc8(AD_BEACON_LEFT);
		debug[1] = get_adc8(AD_BEACON_RIGHT);
		debug[2] = get_adc8(AD_POT);

		debug[3] = line.data[7];		
		debug[4] = line.data[0]	;	
		debug[5] = get_adc8(AD_MOTOR_CURRENT);
		
		timer10ms = 15;
		while(timer10ms > 0);
		refreshDebug();
		}

}

*/
//Switch position = D

void test_line_photocells()  
{  
	while(1 == 1){
		refreshLineData();
		refresh_sensors();
		refreshEdgeSensors();
		debug[0] = line.brightness;  //Varies from 0x80 (black) to 0xd0 (white)
		debug[1] = edge_left;		//Table edge left
		debug[2] = edge_right;		
		debug[3] = line.contrast;
		debug[4] =  line.centroid & 255;
		debug[5] =  line.map;
		
		refreshDebug();
	}
}


//Switch position E
void test_first_6_photocells()
{ uint8_t i;
while(1 == 1){
		refreshLineData();
		refresh_sensors();
		for(i=0;i<6;i++) debug[i] = line.data[i];
		
		refreshDebug();
	}
}


//--------------------------------------------------------

//Switch pos 8

//Save photocell white levels in EEPROM
//This must be done at least once.
//Bot must be on a white surface with no line!


int saveLineWhiteLevels()
{
    int i;
	uint8_t adr, cksum;
	
	delay_start(500,whitelevel);  //5 sec delay
	cksum = 0;
	//Read photocells.  Assume we are on pure white with no black line 
	for(adr=0;adr<8;adr++){
		if(adr & 1) sbi(PORTC,0); else cbi(PORTC,0);
		if(adr & 2) sbi(PORTC,1); else cbi(PORTC,1);
		if(adr & 4) sbi(PORTC,2); else cbi(PORTC,2);
		line.data[adr] = get_adc8(AD_LINE_FOLLOW);
		
		}
		
	line.data[8] = checksum(line.data,8);
	
     
   eeprom_write_block (line.data, (void *)0, 9) ; //Save values in EEPROM
   eeprom_write_block (line.data, (void *)16, 9) ; //Save values in EEPROM backup location
   
   eeprom_read_block((void*)&line.data, (const void*)0, 9);  //Read back for verification
   
   
   if(checksum(line.data,8) != line.data[8]) return -1;
      
   for(i=0;i<6;i++) debug[i] = line.data[i];
   refreshDebug();								//Show first 6 values on LCD
   run_seconds = 0;
   while(run_seconds < 5);
   return 0;
   


} 

//-----------------------------------------------------------


// Switch position = C

/*void test_quadrature()

{   int d;
	
	odo_right = 0;
	odo_left = 0;
	delay_start(500,(PGM_P)testquad);  //5 sec delay
	timer_trip = 200;
	d = 0;
	
	while(1 == 1){
		refresh_sensors();
		speed = pot - 128;
		if(timer_trip == 0) speed = 0;
		l_speed = speed;
		r_speed = speed;
		d = odo_right - odo_left;
		if(d > 10) d = 10;
		if(d < -10) d = -10;
		
		l_speed = l_speed + d;
		r_speed = r_speed - d;
		set_motor_speed(l_speed, r_speed);
		
		if(debugReady()){
			debug[0] = (odo_right >> 16) & 255;
			debug[1] = (odo_right >> 8) & 255;
			debug[2] = (odo_right & 255);
			debug[3] = (odo_left >> 16) & 255;
			debug[4] = (odo_left >> 8) & 255;
			debug[5] = (odo_left & 255);
			refreshDebug();
		}
	}
		
		
	
} */

//----------------------------------------------------------------------------
void showStack()
{

if(debugReady()){
			debug[0] = SPH ;
			debug[1] = SPL;
			debug[2] = 0;
			debug[3] = 0;
			debug[4] = 0;
			debug[5] = 0;
			refreshDebug();
		}
		
		while(1 == 1);
		
}

//----------------------------------------------------------------------------

void glitchTest()
{
   int glitchCount1 = 0;
   int glitchCount2 = 0;

	while(1 == 1){
		refreshIRdata();
		if(edge_left < 0x20) glitchCount1++;
		if(edge_right < 0x20) glitchCount2++;
		if(debugReady()){
			debug[0] = glitchCount1 >> 8;
			debug[1] = glitchCount1 & 255;
			debug[2] = edge_left;
			debug[3] = glitchCount2 >> 8;
			debug[4] = glitchCount2 & 255;
			debug[5] = edge_right;
			refreshDebug();
		}
		
		while(timer100us > 0);
		timer100us = 30;
	}

	}	

//-------------------------

   
   

void drivetest()
{

   driveStraight(2000,60);


}		
//---------------------------------------------------------------------------

int main(void)
{ 
  char *sbuf;
  int last_sec, rv;
   
/* Digital ports setup */
   
  PORTD = 0x80;			//PORTD setup
  DDRD = 0x32;
              
  PORTB = 0xff;			// PORTB:  pull ups for selector switch 
  DDRB = 0x10;			// 0..3 inputs for octal selector switch. Bit 4 output for motor enable 
  
  PORTC = 0xff;
  DDRC = 0xff;			//PORTC all outputs
	

/* PORTA: A/D Converter setup */

  DDRA = 0;					// 8 bits analog inputs	
  PORTA = 0;				// Clear port A  
  ADCSRA = BV(ADEN) + BV(ADPS2) + BV(ADPS1); // Enable A to D converter with 16mhz/64 conv clk.  ( 250 KHZ)
  
/* Timer 1 dual PWM setup */

  TCCR0B = BV(CS01);		        // use CLK/8 source for counter 
  TCCR1A = BV(COM1B1) + BV(COM1A1) + BV(WGM10);  // 8 bit pwm  OCnx clear at top
  TCCR1B = BV(CS10);				// clk/1 prescale, 8 bit phase correct pwm  (28.8 khz pwm freq))  
  TCCR1C = BV(FOC1A);				// Force output compare for channel A
  TIMSK1 = 0;						// No interrupts from this timer/coounter
  OCR1AH = 0;						// Clear all output compare registers
  OCR1AL = 128;
  OCR1BH = 0;
  OCR1BL = 128;
  
  

/* Timer-Counter 2 setup */
  TCCR2A = BV(WGM21);			//CTC mode
  TCCR2B = BV(CS21) + BV(CS20);	//Prescale = 32  on TC2 (500 khz with 16mhz osc)  
  OCR2A = 49;					// T/C 2 compare register A for 10,000 hz interrupts (clk/(32*(1+OCR2A))  
  sbi(TIMSK2,OCIE2A);           // Enable timer 2 compare A interrupts

/* UART setup */
  UBRR0L = 103;              /* UART 0 baud rate set to 9600 (16mhz osc) */
  sbi(UCSR0B,TXEN0);              // enable uart 0 transmitter
	
/*External interrupt setup*/
	EICRA = BV(ISC11) + BV(ISC01) ;	//Falling edge interrupts on INT0 and INT1
	sbi(EIMSK,INT1);				//Enable INT1
	sbi(EIMSK,INT0);				//Enable INT0
	

  trip_counter = 0;
  txstate = 0;
  minutes = 0;
  run_seconds = 0;
  last_sec = 0xffff;
  last_sec = 0;
  
  odo_left = 0;
  odo_right = 0;
  line_history_odo_right = 0;
  line_history_odo_left = 0;
  
  l_speed = 0;
  r_speed = 0;
  timer10ms = 500;				//Set for 5 second delay
  timer_overcurrent = 100;		//1000ms
  sel_sw = (PINB ^ 0x0f) & 0x0f;		//Read selector switch
  
  
  sei();                      /* enable global interrupts */
  
 // wdt_enable(WDTO_250MS);          /* enable watchdog timer - 250 ms timeout. */ 
	
	cbi(PORTC,7);	//Test LED off
    sbi(PORTC,3);   //IR LEDs off
	cbi(PORTB,MOT_ENAB);   //Disable motors
	set_motor_speed(0,0);
	
	//line follower PID defaults
	pid_line.kp = 2;		//porportional times 2
	pid_line.ki = 200;		//Integral div by 200
	pid_line.kd = 120;		//Deriviative times 120
	pid_line.t = 10;		//10ms between updates
	pid_line.ilim = 100;	//Limit integral to +/- 100
	pid_line.i = 0;			//Clear all the data
	pid_line.d = 0;
	pid_line.p = 0;
	pid_line.u = 0;
	
	
  delay(10);			//Wait 100ms for LCD to init.	
  clearScreen();
   
  rv = 0;
  
  switch (sel_sw){			//Run job based on selector switch position
	  case 0:	lineFollow(); break;
	  case 1:	rv = beaconFinder(); break;
	  case 2:	deadReckoning(); break;
	  case 3:	bulldozer(); break;
	  case 4:	lineMaze(RIGHT); break;
	  case 5:	lineMaze(LEFT); break;
	 case 6:	glitchTest(); break;
	 case 7:    drivetest(); break;
	 case 0x08:	rv = saveLineWhiteLevels(); break;    //White level calibration
	 case 0x09:	showStack(); break;
	 case 0x0a:	test_ir(); break;
	 case 0x0b:   test_beacon();  break;    //Show beacon sensor data on debug LCD
	 case 0x0c:  face_beacon(TRUE); break;  //Test the face beacon function
	   // case 0x0c:	test_quadrature(); break;
	 	  
	  case 0x0d:	test_line_photocells(); break;
	  case 0x0e:   test_first_6_photocells(); break; 	  	  
	 
	  case 0x0f:	test_sensors(); break; 
  
  };
  
 r_speed = l_speed = 0; 
 set_motor_speed(l_speed, r_speed);
 cbi(PORTB,MOT_ENAB);   //Disable motors
 
 clearScreen();
 delay(10);
 if(rv == 0) sbuf = "Job\r\nComplete";
	else sbuf = "Error";
 writelcd(sbuf);
 while(1 == 1);		//Stop here.  Infinite loop.
 return 0;

};

/* End of File */
